//==============================================================================
/*! \file
 * R3Cop\n
 * Copyright (c) 2010 by Rostislav Hulik     \n
 *
 * Author:  Rostislav Hulik, ihulik@fit.vutbr.cz  \n
 * Date:    2011/2/18                          \n
 *
 * 
 * Description:
 * - Class for OpenMesh export into OpenSceneGraph.
 */

#ifndef _OM_GEOMETRY_H_
#define _OM_GEOMETRY_H_

#include <osg/Geometry>
#include <OpenMesh\Core\IO\exporter\ExporterT.hh>

namespace osg
{

/**
 * Class representing an interface between OpenMesh and OSG
 * It's inherited from standart osg::Geometry, co it's easy to add mesh geometry into any osg node
 */
template <class Mesh>
class OSGVisualiser : public Geometry
{
	public:
		/**
		 * Typedef for specifiing additional model attributes 
		 */
		typedef unsigned char AttributeBinding;
		typedef unsigned char Loading;
			
		/**
		 * Normals will be not added into osg
		 */
		static const AttributeBinding NORMAL_NONE = 1;
	
		/**
		 * Normals will be bound per vertex
		 * If there is no normal associated with vertices, they will be computed
		 */
		static const AttributeBinding NORMAL_PER_VERTEX = 2;
	
		/**
		 * Normals will be bound per face
		 * If there is no normal associated with faces, they will be computed
		 */
		static const AttributeBinding NORMAL_PER_FACE = 4;
	
		/**
		 * Color will not be specified in osg 
		 */
		static const AttributeBinding COLOR_NONE = 8;
	
		/**
		 * Color will be bound per vertex
		 * If there is no color associated with vertices, a default color will be added
		 */
		static const AttributeBinding COLOR_PER_VERTEX = 16;
	
		/**
		 * Color will be bound per face
		 * If there is no color associated with faces, a default color will be added
		 */
		static const AttributeBinding COLOR_PER_FACE = 32;
			
		static const Loading LOAD_VERTICES = 1;
		static const Loading LOAD_FACES = 2;
		static const Loading LOAD_NORMALS = 4;
		/**
		 * Constructor - creates an instance of Geometry from an OpenMesh mesh
		 * @param mesh OpenMesh mesh to be converted
		 * @param binding Specifies additional attributes for mesh generation normal binding and color binding. If there is no normal in Mesh, they will be computed
		 * @param defaultColor Specifies default color which will be added, if there is no color attribute in Mesh
		 * @see AttributeBinding
		 */
		OSGVisualiser(Mesh mesh, Loading attributes = LOAD_VERTICES | LOAD_FACES | LOAD_NORMALS, AttributeBinding binding = NORMAL_PER_FACE | COLOR_PER_VERTEX, Vec4 &defaultColor = Vec4(0.5, 0.5, 0.5, 1.0));
	
		/**
		 * Allows testing if geometry was successfully loaded
		 * @return True, if all the model was loaded successfully.
		 */
		bool isLoadedOk();

	private:

		/**
		 * Flag is true if loading was successfull
		 */
		bool loaded_done;

		/**
		 * Loads vertices into osg from OpenMesh
		 * @param mesh Structure with a mesh to be loaded
		 * @return True if there was no problem with adding all the vertices.
		 */
		bool LoadVertices(Mesh mesh);
		bool LoadEdges(Mesh mesh);
	
		/**
		 * Loads faces into osg from OpenMesh
		 * @param mesh Structure with a mesh to be loaded
		 * @return True if there was no problem with adding all the faces.
		 */
		bool LoadFaces(Mesh mesh);
	
		/**
		 * Loads and associes colors into osg from OpenMesh
		 * @param mesh Structure with a mesh to be loaded
		 * @param defaultColor If there is no color in OpenMesh, this color will be associated to every vertex/face
		 * @param binding OSG color binding.
		 * @see AttributeBinding
		 * @return True if there was no problem with adding all the colors.
		 */
		bool LoadColor(Mesh mesh, osg::Vec4 &defaultColor = Vec4(1.0, 1.0, 1.0, 1.0), Geometry::AttributeBinding binding = Geometry::BIND_PER_VERTEX);
	
		/**
		 * Loads and associes normals into osg from OpenMesh (if there is no normals, they will be computed)
		 * @param mesh Structure with a mesh to be loaded
		 * @param binding OSG normal binding.
		 * @see AttributeBinding
		 * @return True if there was no problem with adding all the normals.
		 */
		bool LoadNormals(Mesh mesh, Geometry::AttributeBinding binding = Geometry::BIND_PER_PRIMITIVE);
	}; // class OSGVisualiser

#include <R3OSG/OSGVisualiser.hxx>
}
#endif